@@ -3,6 +3,7 @@ use Mojo::Base 'Mojo::EventEmitter';
 
 use Carp 'croak';
 use DBI;
+use DBD::SQLite;
 use File::Spec::Functions 'catfile';
 use File::Temp;
 use Mojo::SQLite::Database;
@@ -12,7 +13,7 @@ use Scalar::Util 'weaken';
 use URI;
 use URI::db;
 
-our $VERSION = '1.001';
+our $VERSION = '1.002';
 
 has 'auto_migrate';
 has database_class  => 'Mojo::SQLite::Database';
@@ -82,11 +83,15 @@ sub from_string {
 sub _dequeue {
   my $self = shift;
   while (my $dbh = shift @{$self->{queue} || []}) { return $dbh if $dbh->ping }
-  my $dbh = DBI->connect($self->dsn, undef, undef, $self->options);
-  if (defined $dbh) {
-    $dbh->do('pragma journal_mode=WAL');
-    $dbh->do('pragma synchronous=NORMAL');
-  }
+  my $dbh = DBI->connect($self->dsn, undef, undef, $self->options)
+    // croak "DBI connection to @{[$self->dsn]} failed: $DBI::errstr"; # RaiseError disabled
+  $dbh->do('pragma journal_mode=WAL');
+  $dbh->do('pragma synchronous=NORMAL');
+  # Cache the last insert rowid on inserts
+  weaken(my $weakdbh = $dbh);
+  $dbh->sqlite_update_hook(sub {
+    $weakdbh->{private_mojo_last_insert_id} = $_[3] if $_[0] == DBD::SQLite::INSERT;
+  });
   ++$self->{migrated} and $self->migrations->migrate
     if !$self->{migrated} && $self->auto_migrate;
   $self->emit(connection => $dbh);
