@@ -1,9 +1,8 @@
 package Mojo::SQLite::Database;
 use Mojo::Base 'Mojo::EventEmitter';
 
-use Carp 'croak';
+use Carp qw(croak shortmess);
 use DBI 'SQL_VARCHAR';
-use DBD::SQLite;
 use Mojo::IOLoop;
 use Mojo::JSON 'to_json';
 use Mojo::SQLite::Results;
@@ -11,7 +10,7 @@ use Mojo::SQLite::Transaction;
 use Mojo::Util 'deprecated';
 use Scalar::Util 'weaken';
 
-our $VERSION = '1.001';
+our $VERSION = '1.002';
 
 our @CARP_NOT = qw(Mojo::SQLite::Migrations);
 
@@ -24,18 +23,6 @@ has notification_poll_interval => sub {
 };
 has results_class              => 'Mojo::SQLite::Results';
 
-sub new {
-  my $self = shift->SUPER::new(@_);
-  # Cache the last insert rowid on inserts
-  if (my $dbh = $self->dbh) {
-    weaken $dbh;
-    $dbh->sqlite_update_hook(sub {
-      $dbh->{private_mojo_last_insert_id} = $_[3] if $_[0] == DBD::SQLite::INSERT;
-    });
-  }
-  return $self;
-}
-
 sub DESTROY {
   my $self = shift;
 
@@ -109,35 +96,42 @@ sub query {
   my $cb = ref $_[-1] eq 'CODE' ? pop : undef;
 
   my $dbh = $self->dbh;
+
+  my $prev_h = $dbh->{HandleError};
+  # Better context for error messages
+  local $dbh->{HandleError} = sub { $_[0] = shortmess $_[0]; ($prev_h and $prev_h->(@_)) ? 1 : 0 };
+
   my ($sth, $errored, $error);
   {
     local $@;
-    eval {
-      $sth = $dbh->prepare_cached($query, undef, 3);
+    unless (eval {
       # If RaiseError has been disabled, we might not get a handle
-      do { _bind_params($sth, @_); $sth->execute } if defined $sth;
+      if (defined($sth = $dbh->prepare_cached($query, undef, 3))) {
+        _bind_params($sth, @_);
+        $sth->execute;
+      }
       1;
-    } or $errored = 1;
-    $error = $@ if $errored;
+    }) { $errored = 1; $error = $@ }
   }
 
-  if ($errored) {
-    # Croak error for better context
-    croak $error unless $cb;
-    $error = $dbh->errstr;
-  }
+  die $error if $errored and !$cb; # bail out for errored "blocking" queries
 
   # We won't have a statement handle if prepare failed in a "non-blocking"
   # query or with RaiseError disabled
-  my $results = defined $sth ? $self->results_class->new(sth => $sth) : undef;
-  $results->{last_insert_id} = $dbh->{private_mojo_last_insert_id} if defined $results;
-  unless ($cb) {
+  my $results;
+  if (defined $sth) {
+    $results = $self->results_class->new(sth => $sth);
+    $results->{last_insert_id} = $dbh->{private_mojo_last_insert_id};
+  }
+
+  unless ($cb) { # blocking
     local $_QUERY_NOTIFICATIONS = 1; # no deprecated message
     $self->_notifications;
     return $results;
   }
 
   # Still blocking, but call the callback on the next tick
+  $error = $dbh->err ? $dbh->errstr : $errored ? ($error || 'Error running SQLite query') : undef;
   Mojo::IOLoop->next_tick(sub { $self->$cb($error, $results) });
   return $self;
 }
@@ -354,14 +348,6 @@ L<Mojo::SQLite> object this database belongs to.
 L<Mojo::SQLite::Database> inherits all methods from L<Mojo::Base> and
 implements the following new ones.
 
-=head2 new
-
-  my $db = Mojo::SQLite::Database->new;
-  my $db = Mojo::SQLite::Database->new(dbh => $dbh, sqlite => Mojo::SQLite->new);
-  my $db = Mojo::SQLite::Database->new({dbh => $dbh, sqlite => Mojo::SQLite->new);
-
-Construct a new L<Mojo::SQLite::Database> object.
-
 =head2 begin
 
   my $tx = $db->begin;
